import bpy
import math
from mathutils import Vector

# Clear existing objects
bpy.ops.object.select_all(action='SELECT')
bpy.ops.object.delete()

# Star data: [name, distance (light years), x, y, z coordinates (in light years)]
# Extended to 20 light years with adjusted positions for better visibility
nearby_stars = [
    ["Sun", 0, 0, 0, 0],
    ["Alpha Centauri System", 4.3650, -2.5, 1.5, -3.64, [
        ("Alpha Centauri A", 0.2, 0.1, 0),
        ("Alpha Centauri B", -0.2, -0.1, 0),
        ("Proxima Centauri", 0, 0.15, 0.1)
    ]],
    ["Barnard's Star", 5.9630, -0.24, -5.70, 1.55],
    ["Wolf 359", 7.9000, 2.41, -6.97, -2.45],
    ["Lalande 21185", 8.3040, -1.94, -7.75, 2.85],
    ["Sirius System", 8.6000, -3.5, -8.5, -4.13, [
        ("Sirius A", 0.15, 0, 0),
        ("Sirius B", -0.15, 0, 0)
    ]],
    ["Luyten 726-8 System", 8.7280, 2.36, -8.11, 2.37, [
        ("Luyten 726-8 A", 0.15, 0, 0),
        ("Luyten 726-8 B", -0.15, 0, 0)
    ]],
    ["Ross 154", 9.6800, 3.86, -8.51, -2.40],
    # Additional stars up to 20 light years
    ["Ross 248", 10.32, 2.95, 9.01, 3.85],
    ["Epsilon Eridani", 10.50, -6.85, -7.89, 0.63],
    ["Lacaille 9352", 10.74, -4.60, -9.21, -2.96],
    ["Ross 128", 11.01, -0.59, 10.64, 2.73],
    ["EZ Aquarii System", 11.10, 1.46, -9.89, 4.87, [
        ("EZ Aquarii A", 0.1, 0, 0),
        ("EZ Aquarii B", -0.1, 0.1, 0),
        ("EZ Aquarii C", -0.1, -0.1, 0)
    ]],
    ["Procyon System", 11.40, -6.62, -9.26, -1.30, [
        ("Procyon A", 0.2, 0, 0),
        ("Procyon B", -0.2, 0, 0)
    ]],
    ["61 Cygni System", 11.40, 4.98, 9.86, 3.21, [
        ("61 Cygni A", 0.15, 0, 0),
        ("61 Cygni B", -0.15, 0, 0)
    ]],
    ["Struve 2398", 11.64, 7.45, 8.76, -1.76],
    ["Groombridge 34", 11.73, 8.65, 7.91, -0.52],
    ["Epsilon Indi", 11.80, -2.21, -10.12, -5.59],
    ["DX Cancri", 11.83, -4.37, 10.89, 1.41],
    ["Tau Ceti", 11.90, -7.93, -8.81, -0.89],
    ["Gliese 1061", 12.04, -3.48, -10.97, -3.46],
    ["YZ Ceti", 12.10, -8.48, -8.63, -0.64]
]

# Material colors for different star types with increased emission strength
def create_star_material(name, color, emission_strength=5.0):
    material = bpy.data.materials.new(name=name)
    material.use_nodes = True
    nodes = material.node_tree.nodes
    nodes.clear()
    
    emission = nodes.new(type='ShaderNodeEmission')
    emission.inputs[0].default_value = color
    emission.inputs[1].default_value = emission_strength
    
    material_output = nodes.new(type='ShaderNodeOutputMaterial')
    links = material.node_tree.links
    links.new(emission.outputs[0], material_output.inputs[0])
    
    return material

# Create materials for different star types
sun_material = create_star_material("Sun_material", (1, 0.95, 0.8, 1), 8.0)
orange_star_material = create_star_material("Orange_star_material", (1, 0.7, 0.3, 1), 6.0)
red_dwarf_material = create_star_material("Red_dwarf_material", (1, 0.4, 0.4, 1), 4.0)
white_dwarf_material = create_star_material("White_dwarf_material", (0.9, 0.95, 1, 1), 6.0)

def create_star(name, location, size, material):
    bpy.ops.mesh.primitive_uv_sphere_add(radius=size, location=location)
    star = bpy.context.active_object
    star.name = name
    
    bpy.ops.object.shade_smooth()
    
    subsurf = star.modifiers.new(type='SUBSURF', name='Subdivision')
    subsurf.levels = 2
    subsurf.render_levels = 3
    
    if star.data.materials:
        star.data.materials[0] = material
    else:
        star.data.materials.append(material)
    
    return star

# Create a scale factor
scale_factor = 0.5

# Create stars with system grouping
for star_data in nearby_stars:
    name = star_data[0]
    base_location = Vector((star_data[2], star_data[3], star_data[4])) * scale_factor
    
    if len(star_data) > 5:  # This is a star system
        bpy.ops.object.empty_add(type='SPHERE', location=base_location)
        system = bpy.context.active_object
        system.name = name
        
        for component in star_data[5]:
            comp_name, offset_x, offset_y, offset_z = component
            offset = Vector((offset_x, offset_y, offset_z))
            comp_location = base_location + offset
            
            # Determine star type and size
            if "A" in comp_name:
                if "Procyon" in comp_name or "61 Cygni" in comp_name:
                    size = 0.4
                    material = orange_star_material
                else:
                    size = 0.45
                    material = sun_material
            elif "B" in comp_name:
                if "Procyon" in comp_name or "Sirius" in comp_name:
                    size = 0.25
                    material = white_dwarf_material
                else:
                    size = 0.35
                    material = red_dwarf_material
            else:  # Additional components (C, etc.)
                size = 0.3
                material = red_dwarf_material
            
            star = create_star(comp_name, comp_location, size, material)
            star.parent = system
    else:  # Single star
        if name == "Sun":
            size = 0.5
            material = sun_material
        elif name in ["Epsilon Eridani", "Tau Ceti"]:
            size = 0.4
            material = orange_star_material
        else:
            size = 0.3
            material = red_dwarf_material
        
        create_star(name, base_location, size, material)

# Create 20 light-year reference sphere
bpy.ops.mesh.primitive_uv_sphere_add(radius=20 * scale_factor)
reference_sphere = bpy.context.active_object
reference_sphere.name = "20_Light_Year_Sphere"

# Set up reference sphere material
reference_material = bpy.data.materials.new(name="Reference_Sphere_Material")
reference_material.use_nodes = True
reference_material.blend_method = 'BLEND'
nodes = reference_material.node_tree.nodes
nodes.clear()

transparent = nodes.new(type='ShaderNodeBsdfTransparent')
emission = nodes.new(type='ShaderNodeEmission')
mix = nodes.new(type='ShaderNodeMixShader')
material_output = nodes.new(type='ShaderNodeOutputMaterial')

emission.inputs[0].default_value = (0.5, 0.5, 1, 1)
emission.inputs[1].default_value = 0.1

mix.inputs[0].default_value = 0.9
links = reference_material.node_tree.links
links.new(transparent.outputs[0], mix.inputs[1])
links.new(emission.outputs[0], mix.inputs[2])
links.new(mix.outputs[0], material_output.inputs[0])

reference_sphere.data.materials.append(reference_material)
reference_sphere.display_type = 'WIRE'

# Create text labels
def create_text(name, location, below=False):
    bpy.ops.object.text_add(location=location)
    text = bpy.context.active_object
    text.data.body = name
    text.data.size = 0.25
    
    text_material = bpy.data.materials.new(name=f"Text_Material_{name}")
    text_material.use_nodes = True
    nodes = text_material.node_tree.nodes
    nodes.clear()
    
    emission = nodes.new(type='ShaderNodeEmission')
    emission.inputs[0].default_value = (1, 1, 1, 1)
    emission.inputs[1].default_value = 2.0
    
    material_output = nodes.new(type='ShaderNodeOutputMaterial')
    text_material.node_tree.links.new(emission.outputs[0], material_output.inputs[0])
    
    text.data.materials.append(text_material)
    text.rotation_euler = (math.pi/2, 0, 0)
    
    return text

# Create labels
for star_data in nearby_stars:
    name = star_data[0]
    location = Vector((star_data[2], star_data[3], star_data[4])) * scale_factor
    
    # Adjust label position based on star system
    if name == "Ross 154":
        text_location = location + Vector((0, 0, -0.4))
    elif name in ["EZ Aquarii System", "61 Cygni System", "Procyon System", "Sirius System", "Luyten 726-8 System"]:
        text_location = location + Vector((0, 0, 0.25))  # Closer to the spheres
    else:
        text_location = location + Vector((0, 0, 0.4))
    
    create_text(name, text_location)

# Set up camera with adjusted position for wider view
bpy.ops.object.camera_add(location=(25, -25, 15))
camera = bpy.context.active_object
camera.rotation_euler = (math.pi/4, 0, math.pi/4)

# Set up lighting
bpy.ops.object.light_add(type='SUN', location=(20, 20, 20))
sun_light = bpy.context.active_object
sun_light.data.energy = 8

bpy.ops.object.light_add(type='AREA', location=(-10, -10, -10))
ambient_light = bpy.context.active_object
ambient_light.data.energy = 3

# Set active camera
bpy.context.scene.camera = camera

# Set up world background
world = bpy.context.scene.world
if world is None:
    world = bpy.data.worlds.new("World")
    bpy.context.scene.world = world

world.use_nodes = True
bg = world.node_tree.nodes["Background"]
bg.inputs[0].default_value = (0, 0, 0, 1)

# Render settings
bpy.context.scene.render.engine = 'CYCLES'
bpy.context.scene.cycles.device = 'GPU'
bpy.context.scene.render.resolution_x = 1920
bpy.context.scene.render.resolution_y = 1080
bpy.context.scene.cycles.samples = 256
bpy.context.scene.cycles.preview_samples = 32

# Update viewport
bpy.context.view_layer.update()
